/*
def MACRO key_code

set MACRO command
meta MACRO command

*.key file example
-------------------
#comment
def KEY_UP 24
def KEY_DOWN 2

set KEY_UP <up>
meta KEY_LEFT <history>
*/

#include "keybind.h"
#include "StringTokenizer.h"
#include "define.h"
#include "strutil.h"
#include <termios.h>

#include <iostream>
#include <fstream>

using namespace std;
using namespace strutil;

namespace MLS {

/// @brief		파일에서 바인딩 파일을 읽어들입니다.
/// @param fn   읽어올 파일
/// @return		SUCCESS or ERROR.
bool KeyBind::Load( const string &fn	)
{
	string   line;	
	ifstream in(fn.c_str());
	
	if (!in) return false;
	
	while(getline(in, line))
	{
		if (line.empty() || line[0] == '#')
		{
			continue;
		}

		StringTokenizer st(line, "\t\r\n ");
	
		if (!st.Next())
		{
			continue;
		}

		string cmd = st.Get();
		
		if (!st.Next())
		{
			continue;
		}
		string macro = strutil::tolower(st.Get());
		
		string param = chop(st.Rest());
		
		if (param.empty())
		{
			continue;
		}
				
		if (cmd == "def")
		{
			// backspace 확장
			if (macro == "backspace") 
			{
				struct termios mode;
				memset(&mode, 0, sizeof(mode));
				
				tcgetattr(0, &mode);							
				_definitions[macro] = mode.c_cc[VERASE];
			}
			else
			{
				_definitions[macro] = atoi(param.c_str());
			}
		}
		else if (cmd == "set")
		{
			map<string, int>::iterator i;
			
			if (macro.size() >= 3 && macro[0]=='\'' && macro[2]=='\'') 
			{
				_bindings[(int)macro[1]] = param;				
			}
			else if ((i = _definitions.find(macro)) != _definitions.end())
			{
				_bindings[(*i).second] = param;
			}
		}
		else if (cmd == "meta")
		{
			map<string, int>::iterator i;

			if (macro.size() >= 3 && macro[0]=='\'' && macro[2]=='\'') 
			{			
				_meta_bindings[(int)macro[1]] = param;				
			}
			else if ((i = _definitions.find(macro)) != _definitions.end())
			{
				_meta_bindings[(*i).second] = param;
			}
		}		
	}	
	in.close();
	return true;
}

static string EmptyString;

int KeyBind::GetDefinition(const string &var)
{
	map<string, int>::iterator i;
	
	if ( (i = _definitions.find(tolower(var))) != _definitions.end() ) return (*i).second;
	
	return -1;
}

string KeyBind::GetCommand( int key )
{
	map<int, string>::iterator i;

	if ( (i = _bindings.find(key)) != _bindings.end() ) return (*i).second;
	if ( 'a' <= key && key <= 'z'&& (i = _bindings.find(key-'a'+'A')) != _bindings.end()) return (*i).second;
	if ( 'A' <= key && key <= 'Z'&& (i = _bindings.find(key-'A'+'a')) != _bindings.end()) return (*i).second;

	return EmptyString;
}

string KeyBind::GetMetaCommand( int key )
{
	map<int, string>::iterator i;

	if ( (i = _meta_bindings.find(key)) != _meta_bindings.end() ) return (*i).second;
	if ( 'a' <= key && key <= 'z'&& (i = _meta_bindings.find(key-'a'+'A')) != _meta_bindings.end()) return (*i).second;
	if ( 'A' <= key && key <= 'Z'&& (i = _meta_bindings.find(key-'A'+'a')) != _meta_bindings.end()) return (*i).second;

	return EmptyString;
}

string KeyBind::GetKeyStr(string sCommand)
{
	int 	nKey = 0;
	char	cKeyChar;
	string 	sKeyName;
	map<int, string>::iterator i;
	
	for (i = _meta_bindings.begin(); i != _meta_bindings.end(); ++i)
	{
		if (i->second == sCommand)
		{
			nKey = i->first;
			nKey = nKey + 1000;
		}
	}
	
	for (i = _bindings.begin(); i != _bindings.end(); ++i)
	{
		if (i->second == sCommand)
		{
			nKey = i->first;
		}
	}
	
	if (nKey > 1000)
	{
		cKeyChar = (char)(nKey - 1000);
		sKeyName.append(1, cKeyChar);
		//sKeyName = "Alt+" + strutil::tolower(sKeyName);
		sKeyName = "Alt+" + sKeyName;
	}
	else
	{
		if (nKey != 0 && nKey < 33)
		{
			switch(nKey)
			{
				case 13:
					sKeyName = "ENTER"; 
					break;
				case 27:
					sKeyName = "ESC";
					break;
				default:
					cKeyChar = (char)('A' + nKey - 1);
					sKeyName = "Ctrl+";
					sKeyName.append(1, cKeyChar);
			}
		}
	}
	return sKeyName;
}

KeyBind &KeyBind::GetInstance()
{
	static KeyBind instance;
	return instance;
}

KeyBind::KeyBind( )
{
}

KeyBind::~KeyBind( )
{
}

}; // namespace
